@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Vocabulary::Core;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Implementation of the JSON Schema Core vocabulary
 
-our $VERSION = '0.616';
+our $VERSION = '0.617';
 
 use 5.020;
 use Moo;
@@ -33,7 +33,7 @@ sub evaluation_order ($class) { 0 }
 
 sub keywords ($class, $spec_version) {
   return (
-    '$schema',
+    '$schema',  # must be first to ensure we use the correct Core keywords and subsequent vocabularies
     $spec_version eq 'draft4' ? 'id' : '$id',
     $spec_version !~ /^draft[467]$/ ? '$anchor' : (),
     $spec_version eq 'draft2019-09' ? '$recursiveAnchor' : (),
@@ -60,11 +60,11 @@ sub _traverse_keyword_id ($class, $schema, $state) {
 
   if (length $uri->fragment) {
     return E($state, '%s value "%s" cannot have a non-empty fragment', $state->{keyword}, $schema->{$state->{keyword}})
-      if $state->{spec_version} !~ /^draft[467]$/;
+      if $state->{specification_version} !~ /^draft[467]$/;
 
     if (length(my $base = $uri->clone->fragment(undef))) {
       return E($state, '$id cannot change the base uri at the same time as declaring an anchor')
-        if $state->{spec_version} =~ /^draft[67]$/;
+        if $state->{specification_version} =~ /^draft[67]$/;
 
       # only permitted in draft4: add an id and an anchor via the single 'id' keyword
       return if not $class->__create_identifier($base, $state);
@@ -91,16 +91,13 @@ sub __create_identifier ($class, $uri, $state) {
   $state->{traversed_schema_path} = $state->{traversed_schema_path}.$state->{schema_path};
   $state->{schema_path} = '';
 
-  # Note that even though '$id' is considered ahead of '$schema' in the keyword list, we have
-  # already parsed the '$schema' keyword (before we even started looping through all vocabularies)
-  # and therefore this data (specification_version and vocabularies) is known to be correct.
+  # Note that since '$schema' is considered ahead of '$id' in the keyword list, the dialect
+  # (specification_version and vocabularies) is known to be correct.
 
   $state->{identifiers}{$uri} = {
     path => $state->{traversed_schema_path},
     canonical_uri => $uri,
-    specification_version => $state->{spec_version}, # note! $schema keyword can change this
-    vocabularies => $state->{vocabularies}, # reference, not copy
-    configs => $state->{configs},
+    $state->%{qw(specification_version vocabularies)},
   };
 
   return 1;
@@ -110,26 +107,25 @@ sub _eval_keyword_id ($class, $data, $schema, $state) {
   # we already indexed the anchor uri, so there is nothing more to do at evaluation time.
   # we explicitly do NOT set $state->{initial_schema_uri} or change any other $state values.
   return 1
-    if $state->{spec_version} =~ /^draft[467]$/ and $schema->{$state->{keyword}} =~ /^#/;
+    if $state->{specification_version} =~ /^draft[467]$/ and $schema->{$state->{keyword}} =~ /^#/;
 
   my $schema_info = $state->{evaluator}->_fetch_from_uri($state->{initial_schema_uri}->clone->fragment($state->{schema_path}));
 
   # this should never happen, if the pre-evaluation traversal was performed correctly
   abort($state, 'failed to resolve "%s" to canonical uri', $state->{keyword}) if not $schema_info;
 
+  # $state->{document} is set by evaluate() and does not change unless following a reference
   abort($state, 'EXCEPTION: mismatched document when processing %s "%s"',
       $state->{keyword}, $schema->{$state->{keyword}})
     if $schema_info->{document} != $state->{document};
 
+  # these will all be set when we are at the document root, or if we are here via a $ref,
+  # but not if we are organically passing through this subschema.
   $state->{initial_schema_uri} = $schema_info->{canonical_uri};
-  # these will already be set in all cases: at document root, or if we are here via a $ref
   $state->{traversed_schema_path} = $state->{traversed_schema_path}.$state->{schema_path};
   $state->{schema_path} = '';
-  # these will already be set if there is an adjacent $schema keyword, or if we are here via a $ref
-  $state->{spec_version} = $schema_info->{specification_version};
-  $state->{vocabularies} = $schema_info->{vocabularies};
+  $state->@{qw(specification_version vocabularies)} = $schema_info->@{qw(specification_version vocabularies)};
 
-  $state->@{keys $state->{configs}->%*} = values $state->{configs}->%*;
   push $state->{dynamic_scope}->@*, $state->{initial_schema_uri};
 
   return 1;
@@ -187,16 +183,15 @@ sub _traverse_keyword_schema ($class, $schema, $state) {
     if exists $schema->{'$ref'} and $spec_version =~ /^draft[467]$/;
 
   $state->{evaluator}->_set_metaschema_vocabulary_classes($schema->{'$schema'}, [ $spec_version, $vocabularies ]);
-  $state->@{qw(spec_version vocabularies metaschema_uri)} = ($spec_version, $vocabularies, $schema->{'$schema'} =~ s/#$//r);
+  $state->@{qw(specification_version vocabularies metaschema_uri)} = ($spec_version, $vocabularies, $schema->{'$schema'} =~ s/#$//r);
   return 1;
 }
 
 sub _eval_keyword_schema ($class, $data, $schema, $state) {
-  # we can't always rely on metaschema information being set in $state (if we recursed into this
-  # subschema from a parent, where the data is not already set via $ref), and we need to do it now
-  # so the evaluator knows whether to look for an 'id' or an '$id' keyword next (which sets up the
-  # remaining data in $state).
-  $state->@{qw(spec_version vocabularies)} = $state->{evaluator}->_get_metaschema_vocabulary_classes($schema->{'$schema'})->@*;
+  # the dialect can change at any time, even in the middle of a document, where subsequent keywords
+  # and vocabularies can change; however if we came to this schema via a $ref it will already be
+  # set correctly
+  $state->@{qw(specification_version vocabularies)} = $state->{evaluator}->_get_metaschema_vocabulary_classes($schema->{'$schema'})->@*;
   return 1;
 }
 
@@ -205,13 +200,13 @@ sub _traverse_keyword_anchor ($class, $schema, $state) {
 
   my $anchor = $schema->{$state->{keyword}};
   return E($state, '%s value "%s" does not match required syntax', $state->{keyword}, $anchor)
-    if $state->{spec_version} =~ /^draft[467]$/  and $anchor !~ /^#[A-Za-z][A-Za-z0-9_:.-]*$/
-      or $state->{spec_version} eq 'draft2019-09' and $anchor !~ /^[A-Za-z][A-Za-z0-9_:.-]*$/
-      or $state->{spec_version} eq 'draft2020-12' and $anchor !~ /^[A-Za-z_][A-Za-z0-9._-]*$/;
+    if $state->{specification_version} =~ /^draft[467]$/  and $anchor !~ /^#[A-Za-z][A-Za-z0-9_:.-]*$/
+      or $state->{specification_version} eq 'draft2019-09' and $anchor !~ /^[A-Za-z][A-Za-z0-9_:.-]*$/
+      or $state->{specification_version} eq 'draft2020-12' and $anchor !~ /^[A-Za-z_][A-Za-z0-9._-]*$/;
 
   my $canonical_uri = canonical_uri($state);
 
-  $anchor =~ s/^#// if $state->{spec_version} =~ /^draft[467]$/;
+  $anchor =~ s/^#// if $state->{specification_version} =~ /^draft[467]$/;
   my $uri = Mojo::URL->new->to_abs($canonical_uri)->fragment($anchor);
   my $base_uri = $canonical_uri->clone->fragment(undef);
 
@@ -248,9 +243,7 @@ sub _traverse_keyword_anchor ($class, $schema, $state) {
       # resource index.
       canonical_uri => $base_uri,
       path => $base_path,
-      specification_version => $state->{spec_version},
-      vocabularies => $state->{vocabularies}, # reference, not copy
-      configs => $state->{configs},
+      $state->%{qw(specification_version vocabularies)},
       anchors => {
         $anchor => {
           canonical_uri => $canonical_uri,
@@ -285,7 +278,10 @@ sub _eval_keyword_recursiveAnchor ($class, $data, $schema, $state) {
   return 1;
 }
 
-*_traverse_keyword_dynamicAnchor = \&_traverse_keyword_anchor;
+sub _traverse_keyword_dynamicAnchor ($class, $schema, $state) {
+  return if not $class->_traverse_keyword_anchor($schema, $state);
+  $state->{identifiers}{$state->{initial_schema_uri}}{anchors}{$schema->{'$dynamicAnchor'}}{dynamic} = 1;
+}
 
 # we already indexed the $dynamicAnchor uri, so there is nothing more to do at evaluation time.
 # we explicitly do NOT set $state->{initial_schema_uri}.
@@ -338,11 +334,10 @@ sub _eval_keyword_dynamicRef ($class, $data, $schema, $state) {
     # ...the initial URI MUST be replaced by the URI (including the fragment) for the outermost
     # schema resource in the dynamic scope that defines an identically named fragment with
     # "$dynamicAnchor".
-    foreach my $base_scope ($state->{dynamic_scope}->@*) {
-      my $test_uri = Mojo::URL->new($base_scope)->fragment($anchor);
-      my $dynamic_anchor_subschema_info = $state->{evaluator}->_fetch_from_uri($test_uri);
-      if (defined $dynamic_anchor_subschema_info and ($dynamic_anchor_subschema_info->{schema}{'$dynamicAnchor'}//'') eq $anchor) {
-        $uri = $test_uri;
+    foreach my $scope_uri ($state->{dynamic_scope}->@*) {
+      my $resource = $state->{evaluator}->_get_or_load_resource($scope_uri);
+      if (exists(($resource->{anchors}//{})->{$anchor}) and $resource->{anchors}{$anchor}{dynamic}) {
+        $uri = Mojo::URL->new($scope_uri)->fragment($anchor);
         last;
       }
     }
@@ -412,7 +407,7 @@ JSON::Schema::Modern::Vocabulary::Core - Implementation of the JSON Schema Core
 
 =head1 VERSION
 
-version 0.616
+version 0.617
 
 =head1 DESCRIPTION
 
