@@ -18,10 +18,8 @@ use if $ENV{AUTHOR_TESTING}, 'Test::Warnings';
 use Test::Deep; # import symbols: ignore, re etc
 use Test2::API 'context_do';
 use JSON::Schema::Modern;
-use JSON::Schema::Modern::Utilities 'jsonp';
+use JSON::Schema::Modern::Utilities qw(jsonp true false);
 use Test::File::ShareDir -share => { -dist => { 'JSON-Schema-Modern' => 'share' } };
-use JSON::PP ();
-use constant { true => JSON::PP::true, false => JSON::PP::false };
 
 my $encoder = JSON::Schema::Modern::_JSON_BACKEND()->new
   ->allow_nonref(1)
